/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>
#include <atompub/atom-xml.h>

#include <glib.h>
#include <libxml/tree.h>

struct _atom_feed
{
  GPtrArray *entries;
};

AtomFeed *
atom_feed_new (void)
{
  AtomFeed *feed;
  feed = g_slice_new (AtomFeed);
  feed->entries = g_ptr_array_new ();;
  return feed;
}

void
atom_feed_delete (AtomFeed *feed)
{
  int i;
  for (i = 0; i < feed->entries->len; i++)
    {
      atom_entry_delete (g_ptr_array_index (feed->entries, i));
    }
  g_ptr_array_free (feed->entries, TRUE);
  g_slice_free (AtomFeed, feed);
}

void
atom_feed_entry_append (AtomFeed *feed, AtomEntry *entry)
{
  g_ptr_array_add (feed->entries, entry);
}

void
atom_feed_entry_append_array (AtomFeed *feed, AtomEntry **entries, size_t len)
{
  int i;
  for (i = 0; i < len; i++)
    {
      g_ptr_array_add (feed->entries, entries[i]);
    }
}

xmlNodePtr
atom_feed_to_xmlnode (AtomFeed *feed)
{
  xmlNodePtr node;
  xmlNodePtr entry;
  int i;
  node = xmlNewNode (NULL, "feed");
  xmlNewNs (node, ATOM_NAMESPACE, NULL);
  for (i = feed->entries->len - 1; i >= 0; i--)
    {
      entry = atom_entry_to_xmlnode (g_ptr_array_index (feed->entries, i));
      xmlAddChild (node, entry);
    }
  return node;
}

void
atom_feed_string (AtomFeed *feed, char **buffer, size_t *len)
{
  xmlDocPtr doc;
  xmlNodePtr node;
  doc = xmlNewDoc ("1.0");
  node = atom_feed_to_xmlnode (feed);
  xmlDocSetRootElement (doc, node);
  xmlDocDumpMemory (doc, buffer, len);
  xmlFreeDoc (doc);
}
